@@ -9,9 +9,9 @@ use Perinci::Exporter;
 use Perinci::Object;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2022-09-21'; # DATE
+our $DATE = '2022-09-22'; # DATE
 our $DIST = 'App-FfmpegUtils'; # DIST
-our $VERSION = '0.011'; # VERSION
+our $VERSION = '0.012'; # VERSION
 
 our %SPEC;
 
@@ -58,6 +58,15 @@ sub _nearest {
     sprintf("%d", $_[0]/$_[1]) * $_[1];
 }
 
+sub _convert_pct {
+    my ($num, $total) = @_;
+    if ($num =~ s/%\z//) {
+        $num/100 * $total;
+    } else {
+        $num;
+    }
+}
+
 $SPEC{reencode_video_with_libx264} = {
     v => 1.1,
     summary => 'Re-encode video (using ffmpeg and libx264)',
@@ -269,25 +278,34 @@ $SPEC{split_video_by_duration} = {
     summary => 'Split video by duration into parts',
     description => <<'_',
 
-This utility uses *ffmpeg* (particularly the `-t` and `-ss`) option to split a
+This utility uses **ffmpeg** (particularly the `-t` and `-ss`) option to split a
 longer video into shorter videos. For example, if you have `long.mp4` with
 duration of 1h12m and you run it through this utility with `--every 15min` then
 you will have 5 new video files: `long.1of5.mp4` (15min), `long.2of5.mp4`
 (15min), `long.3of5.mp4` (15min), `long.4of5.mp4` (15min), and `long.5of5.mp4`
 (12min).
 
+Compared to using `ffmpeg` directly, this wrapper offers convenience of
+calculating the times (`-ss`) option for you, handling multiple files,
+automatically choosing output filename, and tab completion.
+
 _
     args => {
         %argspec0_files,
         # XXX start => {},
         every => {
-            schema => 'duration*',
-            req => 1,
+            schema => ['any*', of=>['duration*', 'percent_str*']],
+        },
+        parts => {
+            schema => 'posint*',
         },
         %argspecopt_copy,
         # XXX merge_if_last_part_is_shorter_than => {},
         # XXX output_filename_pattern
     },
+    args_rels => {
+        req_one => [qw/every parts/],
+    },
     examples => [
         {
             summary => 'Split video per 15 minutes',
@@ -303,6 +321,20 @@ _
             test => 0,
             'x.doc.show_result' => 0,
         },
+        {
+            summary => 'Split video into 4 equiduration parts',
+            src_plang => 'bash',
+            src => '[[prog]] foo.mp4 --parts 4',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Another way to split into equiduration parts using percentage in --every',
+            src_plang => 'bash',
+            src => '[[prog]] foo.mp4 --every 25%',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
     ],
     features => {
         dry_run => 1,
@@ -319,8 +351,8 @@ sub split_video_by_duration {
 
     my %args = @_;
     my $files = $args{files};
-    my $part_dur = $args{every};
-    $part_dur > 0 or return [400, "Please specify a non-zero --every"];
+    my $every = $args{every};
+    my $parts = $args{parts};
 
     my $envres = envresmulti();
     my $j = -1;
@@ -342,6 +374,22 @@ sub split_video_by_duration {
             next;
         }
 
+        my $part_dur;
+        if (defined $parts) {
+            return [400, "Please specify a positive number in --parts"] unless $parts >= 1;
+            $part_dur = $total_dur / $parts;
+        } elsif (defined $every) {
+            if ($every =~ s/%\z//) {
+                $every > 0 && $every <= 100 or return [400, "For percentage in --every, please specify a number between 0 and 100"];
+                $part_dur = $total_dur * ($every/100);
+            } else {
+                $every > 0 or return [400, "Please specify a positive number in --every"];
+                $part_dur = $every;
+            }
+        } else {
+            return [400, "Please specify either --every or --parts"];
+        }
+
         my $num_parts = POSIX::ceil($total_dur / $part_dur);
         my $fmt = $num_parts >= 1000 ? "%04d" : $num_parts >= 100 ? "%03d" : $num_parts >= 10 ? "%02d" : "%d";
 
@@ -378,20 +426,70 @@ $SPEC{cut_video_by_duration} = {
     description => <<'_',
 
 This utility uses *ffmpeg* (particularly the `-t` and `-ss`) option to get a
-portion (time range) of a video. It is a convenient wrapper of ffmpeg for this
-particular task. You can specify start time and end time, or start time and
-duration. It automatically chooses a filename if you don't specify one.
+portion (time range) of a video.
+
+Compared to using `ffmpeg` directly, this wrapper offers convenience of more
+flexibility in specifying times and duration (e.g. '15s' as well as '00:10' as
+well as 'PT1M30S'), specifying only 'end' and 'duration', handling multiple
+files, automatically choosing output filename, and tab completion.
 
 _
     args => {
-        %argspec0_file,
+        %argspec0_files,
         start => {
-            schema => 'duration*',
-            req => 1,
-            pos => 1,
+            schema => ['any*', of=>['duration*', 'percent_str*']],
+            cmdline_aliases => {s=>{}},
         },
+        end => {
+            schema => ['any*', of=>['duration*', 'percent_str*']],
+            cmdline_aliases => {e=>{}},
+        },
+        duration => {
+            schema => ['any*', of=>['duration*', 'percent_str*']],
+            cmdline_aliases => {d=>{}},
+        },
+        %argspecopt_copy,
+    },
+    args_rels => {
+        req_one => [qw/start end duration/],
     },
     examples => [
+        {
+            summary => 'Specify start only, the result is 100s.cut_40_to_100.mp4',
+            argv => ['100s.mp4', '-s', '40s'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Specify duration only (using percentage) of two files, the results are 100s.cut_0_to_30.mp4 & 50s.cut_0_to_15.mp4',
+            argv => ['100s.mp4', '50s.mp4', '-d', '30%'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Specify end only (using h:m:s notation), the result is 100s.cut_0_to_63.mp4',
+            argv => ['100s.mp4', '-e', '00:01:03'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Specify start & end only (using h:m:s notation), the result is 100s.cut_50_to_63.mp4',
+            argv => ['100s.mp4', '-s', '00:00:50', '-e', '00:01:03'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Specify start & duration only (using seconds), the result is 100s.cut_50_to_63.mp4',
+            argv => ['100s.mp4', '-s', '50', '-d', '13'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            summary => 'Specify end & duration only (using human & ISO 8601 notation), the result is 100s.cut_50_to_63.mp4',
+            argv => ['100s.mp4', '-e', 'PT63S', '-d', '13s'],
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
     ],
     features => {
         dry_run => 1,
@@ -400,40 +498,100 @@ _
         prog => "ffmpeg", # XXX allow FFMPEG_PATH
     },
     links => [
-        {url=>'prog:srtsplit', summary=>'Split .srt by duration, much like this utility'},
     ],
 };
 sub cut_video_by_duration {
     require POSIX;
 
     my %args = @_;
-    my $file = $args{file};
-    my $part_dur = $args{every};
-    $part_dur > 0 or return [400, "Please specify a non-zero --every"];
+    my $files = $args{files};
+    my $start0    = $args{start};
+    my $end0      = $args{end};
+    my $duration0 = $args{duration};
 
-    require Media::Info;
-    my $res = Media::Info::get_media_info(media => $file);
-    return $res unless $res->[0] == 200;
+    my $envres = envresmulti();
+    my $j = -1;
+    for my $file (@$files) {
+        $j++;
+        require Media::Info;
+        my $res = Media::Info::get_media_info(media => $file);
+        unless ($res->[0] == 200) {
+            $envres->add_result($res->[0], "Can't get media info for video '$file': $res->[1], skipped", {item_id=>$j});
+            next;
+        }
 
-    my $total_dur = $res->[2]{duration}
-        or return [412, "Duration of video is zero"];
+        my $total_dur = $res->[2]{duration}
+            or do {
+                $envres->add_result(412, "Duration of video '$file' is zero, skipped", {item_id=>$j});
+                next;
+            };
 
-    my $num_parts = POSIX::ceil($total_dur / $part_dur);
-    my $fmt = $num_parts >= 1000 ? "%04d" : $num_parts >= 100 ? "%03d" : $num_parts >= 10 ? "%02d" : "%d";
+        my ($start, $end, $duration);
+        if (defined $start0 && !defined $end0 && !defined $duration0) {
+            $start    = _convert_pct($start0, $total_dur);
+            $end      = $total_dur;
+            $duration = $end - $start;
+        } elsif (!defined $start0 && defined $end0 && !defined $duration0) {
+            $start    = 0;
+            $end      = _convert_pct($end0, $total_dur);
+            $duration = $end - $start;
+        } elsif (!defined $start0 && !defined $end0 && defined $duration0) {
+            $start    = 0;
+            $end      = _convert_pct($duration0, $total_dur);
+            $duration = $duration0;
+        } elsif (defined $start0 && defined $end0 && !defined $duration0) {
+            $start    = _convert_pct($start0, $total_dur);
+            $end      = _convert_pct($end0, $total_dur);
+            $duration = $end - $start;
+        } elsif (defined $start0 && !defined $end0 && defined $duration0) {
+            $start    = _convert_pct($start0, $total_dur);
+            $duration = _convert_pct($duration0, $total_dur);
+            $end      = $start + $duration;
+        } elsif (!defined $start0 && defined $end0 && defined $duration0) {
+            $end      = _convert_pct($end0, $total_dur);
+            $duration = _convert_pct($duration0, $total_dur);
+            $start    = $end - $duration;
+        } elsif (defined $start0 && defined $end0 && !defined $duration0) {
+            $start    = _convert_pct($start0, $total_dur);
+            $end      = _convert_pct($end0, $total_dur);
+            $duration = $end - $start;
+        } elsif (defined $start0 && defined $end0 && defined $duration0) {
+            return [409, "Please only specify 'start' & 'end', 'start' & 'duration', or 'end' & 'duration', not all three"];
+        } else {
+            return [500, "Internal bug"];
+        }
 
-    return [304, "No split necessary"] if $num_parts < 2;
+        do { $envres->add_result(400, "Start time ($start) for video '$file' is negative, skipped", {item_id=>$j}); next }   if $start < 0;
+        do { $envres->add_result(400, "End time ($end) for video '$file' is negative, skipped", {item_id=>$j}); next }       if $end < 0;
+        do { $envres->add_result(400, "Non-positive duration ($duration) for video '$file', skipped", {item_id=>$j}); next } if $duration <= 0;
+        do { $envres->add_result(400, "Start time ($start) is larger than end time ($end) for video '$file', skipped", {item_id=>$j}); next } if $end < $start;
+        do { $envres->add_result(400, "Start time ($start) exceeds the total duration of video for video '$file', skipped", {item_id=>$j}); next }  if $start > $total_dur;
+        do { $envres->add_result(400, "End time ($end) exceeds the total duration of video for video '$file', skipped", {item_id=>$j}); next }      if $start > $total_dur;
+        do { $envres->add_result(400, "Duration ($duration) exceeds the total duration of video for video '$file', skipped", {item_id=>$j}); next } if $duration > $total_dur;
 
-    require IPC::System::Options;
-    for my $i (1..$num_parts) {
-        my $part_label = sprintf "${fmt}of%d", $i, $num_parts;
+        if ($start == 0 && $end == $total_dur) {
+            $envres->add_result(304, "No cutting necessary for video '$file' (whole length requested), skipped", {item_id=>$j});
+            next;
+        }
+
+        log_trace "Video %s, start=%s, duration=%s, end=%s", $file, $start, $duration, $end;
+        require IPC::System::Options;
+        my $label = sprintf 'cut_%s_to_%s', $start, $end;
         my $ofile = $file;
-        if ($ofile =~ /\.\w+\z/) { $ofile =~ s/(\.\w+)\z/.$part_label$1/ } else { $ofile .= ".$part_label" }
-        my $time_start = ($i-1)*$part_dur;
+        if ($ofile =~ /\.\w+\z/) { $ofile =~ s/(\.\w+)\z/.$label$1/ } else { $ofile .= ".$label" }
         IPC::System::Options::system(
             {log=>1, dry_run=>$args{-dry_run}},
-            "ffmpeg", "-i", $file, ($args{copy} ? ("-c", "copy") : ()), "-ss", $time_start, "-t", $part_dur, $ofile);
-    }
-    [200];
+            "ffmpeg", "-i", $file, ($args{copy} ? ("-c", "copy") : ()), "-ss", $start, "-t", $duration, $ofile);
+        my ($exit_code, $signal, $core_dump) = ($? < 0 ? $? : $? >> 8, $? & 127, $? & 128);
+        if ($exit_code) {
+            $envres->add_result(500, "ffmpeg exited $exit_code (sig $signal) for video $file: $!", {item_id=>$j});
+        } else {
+            $envres->add_result(200, "Video '$file' successfully cut", {item_id=>$j});
+        }
+
+    } # for $file
+
+    $envres->as_struct;
 }
 
 1;
@@ -451,7 +609,7 @@ App::FfmpegUtils - Utilities related to ffmpeg
 
 =head1 VERSION
 
-This document describes version 0.011 of App::FfmpegUtils (from Perl distribution App-FfmpegUtils), released on 2022-09-21.
+This document describes version 0.012 of App::FfmpegUtils (from Perl distribution App-FfmpegUtils), released on 2022-09-22.
 
 =head1 FUNCTIONS
 
@@ -464,10 +622,43 @@ Usage:
 
 Get a portion (time range) of a video.
 
+Examples:
+
+=over
+
+=item * Specify start only, the result is 100s.cut_40_to_100.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4"], start => "40s");
+
+=item * Specify duration only (using percentage) of two files, the results are 100s.cut_0_to_30.mp4 & 50s.cut_0_to_15.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4", "50s.mp4"], duration => "30%");
+
+=item * Specify end only (using h:m:s notation), the result is 100s.cut_0_to_63.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4"], end => "00:01:03");
+
+=item * Specify start & end only (using h:m:s notation), the result is 100s.cut_50_to_63.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4"], end => "00:01:03", start => "00:00:50");
+
+=item * Specify start & duration only (using seconds), the result is 100s.cut_50_to_63.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4"], duration => 13, start => 50);
+
+=item * Specify end & duration only (using human & ISO 8601 notation), the result is 100s.cut_50_to_63.mp4:
+
+ cut_video_by_duration(files => ["100s.mp4"], duration => "13s", end => "PT63S");
+
+=back
+
 This utility uses I<ffmpeg> (particularly the C<-t> and C<-ss>) option to get a
-portion (time range) of a video. It is a convenient wrapper of ffmpeg for this
-particular task. You can specify start time and end time, or start time and
-duration. It automatically chooses a filename if you don't specify one.
+portion (time range) of a video.
+
+Compared to using C<ffmpeg> directly, this wrapper offers convenience of more
+flexibility in specifying times and duration (e.g. '15s' as well as '00:10' as
+well as 'PT1M30S'), specifying only 'end' and 'duration', handling multiple
+files, automatically choosing output filename, and tab completion.
 
 This function is not exported.
 
@@ -478,9 +669,17 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<file>* => I<filename>
+=item * B<copy> => I<bool>
 
-=item * B<start>* => I<duration>
+Whether to use the "copy" codec (fast but produces inaccurate timings).
+
+=item * B<duration> => I<duration|percent_str>
+
+=item * B<end> => I<duration|percent_str>
+
+=item * B<files>* => I<array[filename]>
+
+=item * B<start> => I<duration|percent_str>
 
 
 =back
@@ -621,13 +820,17 @@ Usage:
 
 Split video by duration into parts.
 
-This utility uses I<ffmpeg> (particularly the C<-t> and C<-ss>) option to split a
+This utility uses B<ffmpeg> (particularly the C<-t> and C<-ss>) option to split a
 longer video into shorter videos. For example, if you have C<long.mp4> with
 duration of 1h12m and you run it through this utility with C<--every 15min> then
 you will have 5 new video files: C<long.1of5.mp4> (15min), C<long.2of5.mp4>
 (15min), C<long.3of5.mp4> (15min), C<long.4of5.mp4> (15min), and C<long.5of5.mp4>
 (12min).
 
+Compared to using C<ffmpeg> directly, this wrapper offers convenience of
+calculating the times (C<-ss>) option for you, handling multiple files,
+automatically choosing output filename, and tab completion.
+
 This function is not exported.
 
 This function supports dry-run operation.
@@ -641,10 +844,12 @@ Arguments ('*' denotes required arguments):
 
 Whether to use the "copy" codec (fast but produces inaccurate timings).
 
-=item * B<every>* => I<duration>
+=item * B<every> => I<duration|percent_str>
 
 =item * B<files>* => I<array[filename]>
 
+=item * B<parts> => I<posint>
+
 
 =back
 
